/*
 * Generic access to real-time clock functionality.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>
#include "rtc-generic.h"

/* Function declarations reproduced here to avoid clock identifier conflicts in
   the chip-specific headers. */

//extern Rtc_chip *jz4730_rtc_chip(l4_addr_t, Cpm_chip *);
//extern Rtc_chip *jz4740_rtc_chip(l4_addr_t, Cpm_chip *);
extern Rtc_chip *jz4780_rtc_chip(l4_addr_t, Cpm_chip *);
extern Rtc_chip *x1600_rtc_chip(l4_addr_t, Cpm_chip *);

struct rtc_function
{
  const char *name;
  Rtc_chip *(*function)(l4_addr_t, Cpm_chip *);
};

static struct rtc_function functions[] = {
  //{"jz4730", jz4730_rtc_chip},
  //{"jz4740", jz4740_rtc_chip},
  {"jz4780", jz4780_rtc_chip},
  {"x1600", x1600_rtc_chip},
  {NULL, NULL}
};

Rtc_chip *new_rtc_chip(const char *name, l4_addr_t start, Cpm_chip *cpm)
{
  for (struct rtc_function *f = functions; f->name != NULL; f++)
  {
    if (!strcmp(name, f->name))
      return f->function(start, cpm);
  }

  return NULL;
}
